@@ -177,7 +177,7 @@ struct slcompress {
 #define SLF_TOSS 1		/* tossing rcvd frames because of input err */
 
 void	 sl_compress_init __P((struct slcompress *, int));
-u_int	 sl_compress_tcp __P((struct mbuf *,
+u_int	 sl_compress_tcp __P((mbuf_t ,
 	    struct ip *, struct slcompress *, int));
 int	 sl_uncompress_tcp __P((u_char **, int, u_int, struct slcompress *));
 int	 sl_uncompress_tcp_core __P((u_char *, int, int, u_int,
